
	name "Pcm Vol"

Vol_L control  #1,0,#1
Vol_R control  #1,0,#1
;;; fadder_L  control  #1,0,#1
;;; fadder_R  control  #1,0,#1

	
front_l	IO
front_r	IO
rear_l  IO
rear_r  IO
center	IO
sub	IO

tmp	dyn
half	con	$40000000
		
	macs  front_l,$40,front_l,Vol_L
	macs  front_r,$40,front_r,Vol_R
	macs  rear_l,$40,rear_l,Vol_L
	macs  rear_r,$40,rear_r,Vol_R
;;; Average both volumes for center/sub
	interp tmp, Vol_L, half, Vol_R
	macs  center,$40,center,tmp
	macs  sub,$40,sub,tmp

	end
	

	
